"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Actions
#Global actions for gui elements to communicate state changes to the action handler.		
#@author Josh Blum

import pygtk
pygtk.require('2.0')
import gtk

######################################################################################################
#	states
######################################################################################################

APPLICATION_INITIALIZE = 'app init'
APPLICATION_QUIT = 'app quit'

SOCKET_SELECT = 'socket select'

NOTHING_SELECT = 'nothing select'

PARAM_MODIFY = 'param modify'

SIGNAL_BLOCK_SELECT = 'signal block select'
SIGNAL_BLOCK_MOVE = 'signal block move'
SIGNAL_BLOCK_ROTATE_LEFT = 'signal block rotate left'
SIGNAL_BLOCK_ROTATE_RIGHT = 'signal block rotate right'
SIGNAL_BLOCK_CREATE = 'signal block create'
SIGNAL_BLOCK_PARAM_MODIFY = 'signal block param modify'
SIGNAL_BLOCK_INC_TYPE = 'signal block increment type'
SIGNAL_BLOCK_DEC_TYPE = 'signal block decrement type'

SOCKET_CONTROLLER_INC = 'socket controller increment'
SOCKET_CONTROLLER_DEC = 'socket controller decrement'

CONNECTION_SELECT = 'connection select'
CONNECTION_CREATE = 'conection create'
ELEMENT_DELETE = 'element delete'

VARIABLE_MODIFY = 'variable modify'
VARIABLE_REORDER = 'variable reorder'

FLOW_GRAPH_OPEN = 'flow graph open'
FLOW_GRAPH_UNDO = 'flow graph undo'
FLOW_GRAPH_REDO = 'flow graph redo'
FLOW_GRAPH_SAVE = 'flow graph save'
FLOW_GRAPH_SAVE_AS = 'flow graph save as'
FLOW_GRAPH_CLOSE = 'flow graph close'
FLOW_GRAPH_NEW = 'flow graph new'
FLOW_GRAPH_WINDOW_RESIZE = 'flow graph window resize'
FLOW_GRAPH_EXEC = 'flow graph exec'
FLOW_GRAPH_STOP = 'flow graph stop'
FLOW_GRAPH_SCREEN_CAPTURE = 'flow graph screen capture'

ABOUT_WINDOW_DISPLAY = 'about window display'
DATA_TYPES_WINDOW_DISPLAY = 'data types window display'
HOTKEYS_WINDOW_DISPLAY = 'hotkeys window display'
USRP_DIAGNOSTICS_DISPLAY = 'usrp diagnostics display'
PREFS_WINDOW_DISPLAY = 'prefs window display'
MATH_EXPR_WINDOW_DISPLAY = 'expression window display'

######################################################################################################
#	actions
######################################################################################################

ACTIONS_LIST = (
	gtk.Action(FLOW_GRAPH_NEW, '_New', 'Create a new flow graph', 'gtk-new'),
	gtk.Action(FLOW_GRAPH_OPEN, '_Open', 'Open a flow graph from file', 'gtk-open'),
	gtk.Action(FLOW_GRAPH_SAVE, '_Save', 'Save this flow graph', 'gtk-save'),
	gtk.Action(FLOW_GRAPH_SAVE_AS, 'Save _As', 'Save the open flow graph as...', 'gtk-save-as'),
	gtk.Action(FLOW_GRAPH_CLOSE, '_Close', 'Close the open flow graph', 'gtk-close'),
	gtk.Action(APPLICATION_QUIT, '_Quit', 'Quit program', 'gtk-quit'),
	gtk.Action(FLOW_GRAPH_UNDO, '_Undo', 'Undo a change to the flow graph', 'gtk-undo'),
	gtk.Action(FLOW_GRAPH_REDO, '_Redo', 'Redo a change to the flow graph', 'gtk-redo'),
	gtk.Action(ELEMENT_DELETE, '_Delete', 'Delete the selected signal block', 'gtk-delete'),
	gtk.Action(SIGNAL_BLOCK_ROTATE_LEFT, 'Rotate _Left', 'Rotate the signal block 90 degrees', 'gtk-go-back'),
	gtk.Action(SIGNAL_BLOCK_ROTATE_RIGHT, 'Rotate _Right', 'Rotate the signal block -90 degrees', 'gtk-go-forward'),
	gtk.Action(SIGNAL_BLOCK_PARAM_MODIFY, '_Properties', 'Modify params for the selected signal block', 'gtk-properties'),
	gtk.Action(FLOW_GRAPH_WINDOW_RESIZE, '_Window Size', 'Set the window size', 'gtk-edit'),
	gtk.Action(USRP_DIAGNOSTICS_DISPLAY, '_USRP Diagnostics', 'Analyze the USRP', 'gtk-dialog-info'),
	gtk.Action(PREFS_WINDOW_DISPLAY, '_Preferences', 'Configure Preferences', 'gtk-preferences'),
	gtk.Action(ABOUT_WINDOW_DISPLAY, '_About', 'About this program', 'gtk-about'),
	gtk.Action(DATA_TYPES_WINDOW_DISPLAY, '_Data Types', 'Data Types', 'gtk-connect'),
	gtk.Action(HOTKEYS_WINDOW_DISPLAY, '_HotKeys', 'Hot Keys', 'gtk-info'),
	gtk.Action(MATH_EXPR_WINDOW_DISPLAY, '_Math Expressions', 'Mathematical Expressions', 'gtk-convert'),
	gtk.Action(FLOW_GRAPH_EXEC, '_Execute', 'Execute the flow graph', 'gtk-execute'),
	gtk.Action(FLOW_GRAPH_STOP, '_Stop', 'Stop the flow graph', 'gtk-stop'),
	gtk.Action(FLOW_GRAPH_SCREEN_CAPTURE, 'S_creen Capture', 'Create a screen capture of the flow graph', 'gtk-print'),
)

ACTIONS_DICT = dict((action.get_name(), action) for action in ACTIONS_LIST)
					
def get_action_from_name(action_name): 
	"""!
	Retrieve the action from the action list.
	Search the list and find an action with said name.
	@param action_name the action name(string)
	@throw KeyError bad action name
	@return a gtk action object
	"""	
	if ACTIONS_DICT.has_key(action_name): return ACTIONS_DICT[action_name]
	raise KeyError('Action Name: "%s" does not exist'%action_name)
	
	