"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Constants
#Global constants
#@author Josh Blum

import sys, os
import mutex

##mutex used when running a flow graph.
MUTEX = mutex.mutex()	
def lock(mutex):
	while not mutex.testandset(): pass	#try to lock repeatedly until lock is aquired
MUTEX.lock = lambda: lock(MUTEX)

######################################################################################################
## Global Titles @{
######################################################################################################

##The current version of this code
VERSION = '0.70'

##The name to appear in the main window for a flow graph that has not been saved to file.
NEW_FLOGRAPH_TITLE = 'untitled'

##The prefix title on the main window.
MAIN_WINDOW_PREFIX = "GRC"
##@}

######################################################################################################
## Signal block connector lengths
######################################################################################################

##The length that a connection must extend from the socket until the length depends on the index of the socket.
CONNECTOR_EXTENSION_INITIAL_LENGTH = 11

##The length that a connection must extend from the initial length times the index of the socket, after this length, the connection may have a bend.
CONNECTOR_EXTENSION_LENGTH = 11

######################################################################################################
## Signal block rotations
######################################################################################################

##List of possible angles (in degrees) that a signal block and its parameters can be rotated to.
POSSIBLE_ROTATIONS = (0, 90, 180, 270)

##direction of rotation left.
DIR_LEFT = 'left'

##direction of rotation right.
DIR_RIGHT = 'right'

######################################################################################################
## Dimension constraints for the various windows (in pixels)
######################################################################################################

##main window constraints	@{
DEFAULT_MAIN_WINDOW_WIDTH = 750

DEFAULT_MAIN_WINDOW_HEIGHT = 550
##@}

##flow graph window constraints	@{
MIN_WINDOW_WIDTH = 600
MAX_WINDOW_WIDTH = 2400

MIN_WINDOW_HEIGHT = 400
MAX_WINDOW_HEIGHT = 1800
##@}

##dialog constraints	@{
MIN_DIALOG_WIDTH = 400
MIN_DIALOG_HEIGHT = 500
##@}

##misc window constraints	@{
REPORTS_WINDOW_HEIGHT = 80

SIGNAL_BLOCK_SELECTION_WINDOW_WIDTH = 250
SIGNAL_BLOCK_SELECTION_WINDOW_HEIGHT = 250

VARIABLE_MODIFICATION_WINDOW_WIDTH = 250
VARIABLE_MODIFICATION_WINDOW_HEIGHT = 150
##@}

######################################################################################################
## Constraints for the sliders in the runnable flow graph @{
######################################################################################################

##The number of tics for a slider.
DEFAULT_SLIDER_STEPS = 100

##The max width that a row of sliders can use.
MAX_SLIDERS_WIDTH = 600

##Slider width in pixels
DEFAULT_SLIDER_WIDTH = 200
##Slider width in pixels
MAX_SLIDER_WIDTH = 300
##Slider width in pixels
MIN_SLIDER_WIDTH = 100

##The height in pixels.
SLIDER_HEIGHT = 20
##@}

######################################################################################################
## Constraints on displayable labels and sockets @{
######################################################################################################

LABEL_SEPARATION = 4
LABEL_PADDING_WIDTH=20
LABEL_PADDING_HEIGHT=10

SOCKET_SEPARATION = 20
SOCKET_HEIGHT = 17
SOCKET_WIDTH = 27
SOCKET_BORDER_SEPARATION = 10

PARAM_FONT = 'Sans 8'
SIGNAL_BLOCK_FONT = 'Sans 9 Bold'
SOCKET_FONT = 'Sans 8'
##@}

######################################################################################################
## Dragging, scrolling, and redrawing constants for the flow graph window in pixels @{
######################################################################################################

##How close can the mouse get to the window border before mouse events are ignored.
BORDER_PROXIMITY_SENSITIVITY = 10

##How close the mouse can get to the edge of the visible window before scrolling is invoked.
SCROLL_PROXIMITY_SENSITIVITY = 30

##When the window has to be scrolled, move it this distance in the required direction.
SCROLL_DISTANCE = 15

##The redrawing sensitivity, how many motion detection events must occur before a redraw?
MOTION_DETECT_REDRAWING_SENSITIVITY = 3

##How close the mouse click can be to a connection and register a connection select.
CONNECTION_SELECT_SENSITIVITY = 5
##@}

######################################################################################################
#	A state is recorded for each change to the flow graph, the size dictates how many states we can record 
######################################################################################################

##The size of the state saving cache in the flow graph (for undo/redo functionality)
STATE_CACHE_SIZE = 42

######################################################################################################
## Constansts dealing with File Paths @{
######################################################################################################

##Location of the python src directory.
SRC_DIR = os.path.abspath(os.path.dirname(__file__))

##Location of external data files.
DATA_DIR = os.path.abspath(SRC_DIR+'/../data/')

##The setting for a blank flow graph.
INITIAL_FLOW_GRAPH_FILE = os.path.abspath(DATA_DIR+'/initial_flow_graph.grc.xml')

##The default file extension for flow graphs.
FLOW_GRAPH_FILE_EXTENSION = '.grc.xml'

##The default file extension for saving flow graph snap shots.
IMAGE_FILE_EXTENSION = '.png'

##The default path for the open/save dialogs.
DEFAULT_FILE_PATH = os.path.expanduser('~')+'/'

##The default icon for the gtk windows.
PY_GTK_ICON = os.path.abspath(DATA_DIR+'/grc-icon-256.png')

##The default icon for the wx windows.
WX_APP_ICON = os.path.abspath(DATA_DIR+'/grc-icon-32.png')
#>>> platform dependency! wx under cygwin has issues with icon paths	#
if sys.platform == 'cygwin': WX_APP_ICON = None

##The default binary to execute python files.
PYEXEC = 'python'
#>>> platform dependency! MacOS requires pythonw to run wx apps	#
if sys.platform == 'darwin': PYEXEC = 'pythonw'

##The default command to run a flow graph file.
DEFAULT_FLOW_GRAPH_EXEC = '%s %s/ExecFlowGraphGUI.py'%(PYEXEC, SRC_DIR)

##The default user preferences file.
PREFERENCES_FILE_PATH = os.path.abspath(DEFAULT_FILE_PATH + '/.grc.xml')
##@}

