"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Elements.GraphicalElement
#Base class for graphical elements such as:
#signal blocks, input sockets, output sockets and connections.
#@author Josh Blum

from Constants import *
import Element
import Colors
import pygtk
pygtk.require('2.0')
import gtk
import pango

class GraphicalElement(Element.Element):
	"""
	GraphicalElement is the base class for all graphical elements. 
	It contains an X,Y coordinate, a list of rectangular areas that the element occupies, 
	and methods to detect selection of those areas.
	"""		
	
	def draw(self, window, BG_color=Colors.BG_COLOR, FG_color=Colors.FG_COLOR):
		"""!
		Draw in the given window.
		@param window the gtk window to draw on
		@param BG_color the background color
		@param FG_color the foreground color
		"""
		gc = self.get_parent().gc	
		self.gc = gc
		X,Y = self.get_coordinate()
		for (rX,rY),(W,H) in self.areas_dict[self.get_rotation()]:
			aX = X + rX
			aY = Y + rY 
			gc.foreground = BG_color
			window.draw_rectangle(gc, True, aX, aY, W, H)
			gc.foreground = self.is_highlighted() and Colors.H_COLOR or FG_color
			window.draw_rectangle(gc, False, aX, aY, W, H)
		for (x1, y1),(x2, y2) in self.lines_dict[self.get_rotation()]:
			gc.foreground = self.is_highlighted() and Colors.H_COLOR or FG_color
			window.draw_line(gc, X+x1, Y+y1, X+x2, Y+y2)

	