"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Elements.GraphicalSocket
#The graphical input/output sockets of the signal block.
#@author Josh Blum

import Utils
import Socket
import GraphicalElement
from DataTypes import *
from Constants import *
import Colors
import pygtk
pygtk.require('2.0')
import gtk
import pango

class GraphicalSocket(Socket.Socket, GraphicalElement.GraphicalElement):
	"""The graphical socket."""	
				
	def update(self):
		"""Create new areas and labels for the socket."""
		self.clear()
		self.BG_color = {
			Complex().get_type():Colors.COMPLEX_COLOR,
			Float().get_type():Colors.FLOAT_COLOR,
			Int().get_type():Colors.INT_COLOR,
			Short().get_type():Colors.SHORT_COLOR,			
			Byte().get_type():Colors.BYTE_COLOR,	
			# and for the vectors	#	
			ComplexVector().get_type():Colors.COMPLEX_VECTOR_COLOR,
			FloatVector().get_type():Colors.FLOAT_VECTOR_COLOR,
			IntVector().get_type():Colors.INT_VECTOR_COLOR,
			ShortVector().get_type():Colors.SHORT_VECTOR_COLOR,			
			ByteVector().get_type():Colors.BYTE_VECTOR_COLOR,
		}[self.get_data_type().get_type()]
		self._create_labels()		
		#	add the input/output area for each rotation angle	#	
		rotation = self.get_rotation()		
		conExtLen = CONNECTOR_EXTENSION_LENGTH
		conExtInitLen = CONNECTOR_EXTENSION_INITIAL_LENGTH
		conOff = SOCKET_HEIGHT/2
		index = self.get_index()
		if Utils.is_input_socket(self): length = self.get_parent().get_num_input_sockets()
		elif Utils.is_output_socket(self): length = self.get_parent().get_num_output_sockets()
		offset = (self.get_parent().H - length*SOCKET_HEIGHT - (length-1)*SOCKET_SEPARATION)/2			
		if rotation == 180 or rotation == 270: index = length-index-1	#reverse the order				
		if (Utils.is_input_socket(self) and rotation == 0) or (Utils.is_output_socket(self) and rotation == 180):		
			x = -1*SOCKET_WIDTH
			y = (SOCKET_SEPARATION+SOCKET_HEIGHT)*index+offset
			self.add_area((x, y), (SOCKET_WIDTH, SOCKET_HEIGHT))
			if self.is_connected(): self.__set_connector_coordinates((x-1, y+conOff),(x-conExtInitLen-conExtLen*index, y+conOff))
		elif (Utils.is_output_socket(self) and rotation == 0) or (Utils.is_input_socket(self) and rotation == 180):	
			x = self.parent.W
			y = (SOCKET_SEPARATION+SOCKET_HEIGHT)*index+offset
			self.add_area((x, y), (SOCKET_WIDTH, SOCKET_HEIGHT))
			if self.is_connected(): self.__set_connector_coordinates((x+1+SOCKET_WIDTH, y+conOff),(x+conExtInitLen+SOCKET_WIDTH+conExtLen*index, y+conOff))
		elif (Utils.is_output_socket(self) and rotation == 90) or (Utils.is_input_socket(self) and rotation == 270):		
			y = -1*SOCKET_WIDTH
			x = (SOCKET_SEPARATION+SOCKET_HEIGHT)*index+offset
			self.add_area((x, y), (SOCKET_HEIGHT, SOCKET_WIDTH))
			if self.is_connected(): self.__set_connector_coordinates((x+conOff, y-1),(x+conOff, y-conExtInitLen-conExtLen*index))
		elif (Utils.is_input_socket(self) and rotation == 90) or (Utils.is_output_socket(self) and rotation == 270):	
			y = self.parent.W
			x = (SOCKET_SEPARATION+SOCKET_HEIGHT)*index+offset
			self.add_area((x, y), (SOCKET_HEIGHT, SOCKET_WIDTH))
			if self.is_connected(): self.__set_connector_coordinates((x+conOff, y+1+SOCKET_WIDTH),(x+conOff, y+SOCKET_WIDTH+conExtInitLen+conExtLen*index))
			
	def _create_labels(self):
		"""Create the labels for the socket."""
		#	create the layout	#
		layout = gtk.DrawingArea().create_pango_layout(self.cname)
		desc = pango.FontDescription(SOCKET_FONT)
		layout.set_font_description(desc) 
		w,h = self.w,self.h = layout.get_pixel_size()	
		#	create the pixmap	#			
		pixmap = gtk.gdk.Pixmap(self.get_parent().get_parent().window, w, h, -1)
		gc = pixmap.new_gc()
		gc.foreground = self.BG_color
		pixmap.draw_rectangle(gc, True, 0, 0, w, h)
		gc.foreground = Colors.TXT_COLOR
		pixmap.draw_layout(gc, 0, 0, layout)
		#	create the images	#
		self.horizontal_label = image = pixmap.get_image(0, 0, w, h)	
		if Utils.is_vertical(self.get_rotation()):	
			self.vertical_label = vimage = gtk.gdk.Image(gtk.gdk.IMAGE_NORMAL, pixmap.get_visual(), h, w)
			for i in range(w):
				for j in range(h): vimage.put_pixel(j, w-i-1, image.get_pixel(i, j))
				
	def draw(self, window):
		"""!
		Draw the socket with a label.
		@param window the gtk window to draw on
		"""
		GraphicalElement.GraphicalElement.draw(self, window, BG_color=self.BG_color)				
		gc = self.gc				
		gc.foreground = Colors.TXT_COLOR
		X,Y = self.get_coordinate()
		(x,y),(w,h) = self.areas_dict[self.get_rotation()][0] #use the first area's sizes to place the labels
		if Utils.is_horizontal(self.get_rotation()): window.draw_image(gc, self.horizontal_label, 0, 0, x+X+(SOCKET_WIDTH-self.w)/2, y+Y+(SOCKET_HEIGHT-self.h)/2, -1, -1)
		elif Utils.is_vertical(self.get_rotation()): window.draw_image(gc, self.vertical_label, 0, 0, x+X+(SOCKET_HEIGHT-self.h)/2, y+Y+(SOCKET_WIDTH-self.w)/2, -1, -1)
	
	def __set_connector_coordinates(self, coor_inner, coor_outer, rotation=None):
		"""!
		Set the coordinates that connectors may attach to.
		The connector coordinates are associated with a rotation. If rotation is not specified, the element's current rotation is used.
		@param coor_inner the corrdinate on the socket
		@param coor_outer the coordinate out of the socket
		@param rotation rotation in degrees
		"""
		if rotation == None: rotation = self.get_rotation()	
		self.connector_coordinates[rotation] = coor_inner, coor_outer	
	
	def get_connector_coordinates(self, rotation=None):
		"""!
		Get the coordinates that Connections may attach to.
		@param rotation rotation in degrees
		@return the connector coordinates ((xin, yin), (xout, yout)) tuple
		"""
		if rotation == None: rotation = self.get_rotation()	
		X,Y = self.get_coordinate()	
		(x1,y1),(x2,y2)= self.connector_coordinates[rotation]
		return (x1+X, y1+Y), (x2+X, y2+Y)
			
	def get_connector_direction(self):
		"""!
		Get the direction that the socket points: 0,90,180,270.
		This is the rotation degree if the socket is an output or
		the rotation degree + 180 if the socket is an input.
		@return the direction in degrees
		"""	
		if Utils.is_output_socket(self): return self.get_rotation()
		elif Utils.is_input_socket(self): return (self.get_rotation() + 180)%360 				
	
class GraphicalInputSocket(GraphicalSocket):
	""" The socket for input	"""
	type = Socket.InputSocket.type

class GraphicalOutputSocket(GraphicalSocket):
	""" The socket for output	"""
	type = Socket.OutputSocket.type
		
		