"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Elements.Utils
#Shared functions for flow graph elements.
#@author Josh Blum

from SignalBlock import SignalBlock
from Socket import OutputSocket,InputSocket
from Connection import Connection	

##	True if the rotation is 90 or 270 degrees.
is_vertical = lambda rot: rot in (90, 270)	

##	True if the rotation is 0 or 180 degrees.
is_horizontal = lambda rot: rot in (0, 180)

def get_angle_from_coordinates((x1,y1), (x2,y2)):
	"""!
	Given two points, calculate the vector direction from point1 to point2, directions are multiples of 90 degrees.
	@param (x1,y1) the coordinate of point 1
	@param (x2,y2) the coordinate of point 2
	@return the direction in degrees
	"""
	if y1 == y2:#0 or 180		
		if x2 > x1: return 0
		else: return 180		
	else:#90 or 270
		if y2 > y1: return 270
		else: return 90

##################################################################
# Test methods for elements
#################################################################
##	test method for signal blocks 
is_signal_block = lambda obj: obj!= None and obj.type == SignalBlock.type
## test methods for sockets @{
is_input_socket = lambda obj: obj!= None and obj.type == InputSocket.type
is_output_socket = lambda obj: obj!= None and obj.type == OutputSocket.type
is_socket = lambda obj: is_input_socket(obj) or is_output_socket(obj)
##@}
##	test method for connections 
is_connection = lambda obj: obj!= None and obj.type == Connection.type

