"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Graphics.SignalBlockParamsDialog
#A dialog for editing a signal block's parameters.
#@author Josh Blum

import pygtk
pygtk.require('2.0')
import gtk
from Dialogs import TextDisplay
from Constants import MIN_DIALOG_WIDTH,MIN_DIALOG_HEIGHT

class SignalBlockParamsDialog(gtk.Dialog):
	"""A dialog box to set signal block parameters."""
	
	def __init__(self, signal_block):
		"""!
		SignalBlockParamsDialog contructor.
		@param signal_block the signal block
		"""
		gtk.Dialog.__init__(self, buttons=('gtk-close', gtk.RESPONSE_CLOSE))
		self.signal_block = signal_block
		self.set_title('Properties: %s'%signal_block.get_id())
		self.set_size_request(MIN_DIALOG_WIDTH, MIN_DIALOG_HEIGHT)
		vbox = gtk.VBox()
		#	Create the title label	#
		label = gtk.Label()
		label.set_markup('\n<b>Parameters: %s</b>\n'%self.signal_block.get_id())
		vbox.pack_start(label, False)
		# Create the scrolled window to hold all the parameters	#
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrolled_window.add_with_viewport(vbox)
		self.vbox.pack_start(scrolled_window, True)
		self.original_data = list()
		#	Add all the parameters	#
		for param in self.signal_block.get_params(): 
			self.original_data.append(param.get_data_type().get_data())
			vbox.pack_start(param.get_input_object(self._handle_changed), False)	
		# Done adding parameters	#								
		if self.signal_block.get_docs():	
			#	Create the title label	#
			label = gtk.Label()
			label.set_markup('\n\n<b>Documentation: %s</b>\n'%self.signal_block.get_cname())
			vbox.pack_start(label, False)	
			#	Create the text box to display notes about the block	#	
			vbox.pack_start(TextDisplay(self.signal_block.get_docs()), False)
		self.show_all()
			
	def _handle_changed(self, param):
		"""!
		A change occured, update any dependent parameters:
		The enum inside the variable type may have changed and, 
		the variable param will need an external update.
		@param param the graphical parameter that initiated the callback		
		"""
		for p in self.signal_block.get_params():
			if hasattr(p.get_data_type(), 'enum_data_type') and \
				param.get_data_type() == getattr(p.get_data_type(), 'enum_data_type'): p.update()
		
	def run(self):
		"""!
		Call run().
		@return true if a change occured.
		"""
		gtk.Dialog.run(self)		
		self.destroy()
		self.data = list()
		for param in self.signal_block.get_params():
			self.data.append(param.get_data_type().get_data())
		return self.original_data != self.data
			