"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Graphics.SignalBlockSelectionWindow
#The signal block selection window gives the user a tree selection to choose a signal block.
#@author Josh Blum

from Constants import *
import pygtk
pygtk.require('2.0')
import gtk
import gobject
from SignalBlockDefs import SB_TREE

class SignalBlockSelectionWindow(gtk.VBox):
	"""The signal block selection window."""
	
	def __init__(self, get_flow_graph):
		"""!
		SignalBlockSelectionWindow constructor.
		Show all possible signal blocks in this dialog. 
		Each signal block is represented by a gtk label of its tag and an add button. 
		The add button tells the flow graph to create the selected block. and add it to the flow graph.
		@param get_flow_graph get the selected flow graph
		"""	
		gtk.VBox.__init__(self)
		self.get_flow_graph = get_flow_graph
		#title label
		label = gtk.Label()
		label.set_markup('<b>Signal Blocks</b>')
		self.pack_start(label, False)		
		#make the tree model for holding blocks	
		self.model = gtk.TreeStore(gobject.TYPE_STRING)
		self.treeview = gtk.TreeView(self.model)		
		self.treeview.set_enable_search(False) #disable pop up search box
		self.treeview.add_events(gtk.gdk.BUTTON_PRESS_MASK)
		self.treeview.connect('button_press_event', self._handle_mouse_button_press)
		selection = self.treeview.get_selection()
		selection.set_mode('single')
		selection.connect('changed', self._handle_selection_change)	
		renderer = gtk.CellRendererText()
		column = gtk.TreeViewColumn("Category", renderer, text=0)
		self.treeview.append_column(column)
		#make the scrolled window to hold the tree view	
		scrolled_window = gtk.ScrolledWindow()
		scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		scrolled_window.add_with_viewport(self.treeview)
		scrolled_window.set_size_request(
			SIGNAL_BLOCK_SELECTION_WINDOW_WIDTH,
			SIGNAL_BLOCK_SELECTION_WINDOW_HEIGHT,
		)
		self.pack_start(scrolled_window)
		#add button
		self.add_button = gtk.Button(None, 'gtk-add')		
		self.add_button.connect('clicked', self._handle_add_button)
		self.pack_start(self.add_button, False)
		#add blocks and categories	
		for category, tags in SB_TREE:
			iter = self.model.insert_before(None, None)
			self.model.set_value(iter, 0, category)
			for tag in tags:
				new_iter = self.model.insert_before(iter, None)							
				self.model.set_value(new_iter, 0, tag[0])
		#initialize
		self._handle_selection_change()
		
	def _handle_mouse_button_press(self, widget, event):
		"""Handle the mouse button press.
		If a left double click is detected, 
		let the handler for the add button decide to add a block."""
		if event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS: self._handle_add_button(widget)
		
	def _handle_selection_change(self, selection=None):
		"""Handle a selection change in the tree view.
		If a selection changes, set the add button sensitive."""
		selection = self.treeview.get_selection()
		model, iter = selection.get_selected()
		if iter != None and not model.iter_has_child(iter): self.add_button.set_sensitive(True)
		else: self.add_button.set_sensitive(False)
	
	def _handle_add_button(self, widget):
		"""Handle the add button clicked signal.
		Add the signal block to the flow graph."""
		if self.get_flow_graph():		
			selection = self.treeview.get_selection()
			model, iter = selection.get_selected()
			if iter != None and not model.iter_has_child(iter): self.get_flow_graph().add_signal_block(model.get_value(iter, 0))
		
		