"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Graphics.USRPDiagnostics
#A dialog for querying USRP subdevices. USRP interfacing methods encapsulated here.
#@author Josh Blum

import pygtk
pygtk.require('2.0')
import gtk
from Actions import USRP_DIAGNOSTICS_DISPLAY,get_action_from_name
from Elements import GraphicalParam
from DataTypes import *
from Dialogs import TextDisplay

def enable_usrp_diagnostics():
	"""Enable the action for USRP diganostics if gnuradio.usrp can be imported."""
	try: 
		from gnuradio import usrp
		get_action_from_name(USRP_DIAGNOSTICS_DISPLAY).set_sensitive(True)
	except ImportError: print "USRP support missing -> USRP diagnostics disabled..."	

class USRPDiagnosticsDialog(gtk.Dialog):
	"""
	The main dialog window for USRP Dignostics. 
	The USRP parameters and feedback will be located inside this dialog.
	"""
	
	def __init__(self):
		"""USRPDiagnosticsDialog contructor.
		Create a new gtk Dialog with a close button, USRP input paramaters, and output labels."""		
		gtk.Dialog.__init__(self, buttons=('gtk-close', gtk.RESPONSE_CLOSE))
		#	Create the title label	#
		self.set_title('USRP Diagnostics')
		self.USRP_number = Int(0, min=0)
		self.USRP_type = Enum([('Receive', 'rx'), ('Transmit', 'tx'),])
		self.USRP_subdev = Enum([
				('Side A:0', (0, 0)), 
				('Side B:0', (1, 0)),
				('Side A:1', (0, 1)), 
				('Side B:1', (1, 1)),
			]
		)
		self.vbox.pack_start(GraphicalParam('Unit Number', self.USRP_number).get_input_object(), False)
		self.vbox.pack_start(GraphicalParam('Transmit/Receive', self.USRP_type).get_input_object(), False)
		self.vbox.pack_start(GraphicalParam('Side:Subdevice', self.USRP_subdev).get_input_object(), False)
		self.diagnose_button = gtk.Button('Query')
		self.diagnose_button.connect('clicked', self._diagnose_usrp)
		self.vbox.pack_start(self.diagnose_button, False)		
		#	Create a text box for USRP queries	#
		self.query_buffer = TextDisplay()
		self.query_buffer.set_text('Press "Query" to retrieve USRP information...')			
		self.vbox.pack_start(self.query_buffer)	
		self.show_all()	
		self.run()
		self.destroy()
		
	def _diagnose_usrp(self, widget=None):
		"""Query the USRP device and copy the results into the query text box."""
		from gnuradio import usrp
		type = self.USRP_type.parse()
		if type == 'rx':	#for the rx query, use the source and rx methods
			make = usrp.source_c
			get_mux = usrp.determine_rx_mux_value
		elif type == 'tx':	#for the tx query, use the sink and tx methods
			make = usrp.sink_c
			get_mux = usrp.determine_tx_mux_value
		try:		
			u = make(self.USRP_number.parse())
			subdev_spec = self.USRP_subdev.parse()	
			subdev = usrp.selected_subdev(u, subdev_spec)#get the subdev
			msg = ">>> USRP Query\n"
			msg = "%s\nName:\n\t%s\n"%(msg,str(subdev.name()))
			msg = "%s\nAutomated Mux:\n\t0x%08x\n"%(msg, 0xFFFFFFFFL & long(get_mux(u, subdev_spec))) #ensure that the value is displayed as: 8 nibbles, unsigned, hex
			msg = "%s\nConverter Rate:\n\t%s\n"%(msg,u.converter_rate())
			msg = "%s\nUses Quadrature:\n\t%s\n"%(msg, str(subdev.is_quadrature()))
			gain_min, gain_max, gain_step = subdev.gain_range()
			msg = "%s\nGain Range (min, max, step size):\n\t%s\n\t%s\n\t%s\n"%(msg, gain_min, gain_max, gain_step)
			freq_min, freq_max, freq_step = subdev.freq_range()
			msg = "%s\nFreq Range (min, max, step size):\n\t%s\n\t%s\n\t%s\n"%(msg, freq_min, freq_max, freq_step)
			self.query_buffer.set_text(msg)
		except Exception, e:	#display the error message
			self.query_buffer.set_text('''\
>>> Error\n%s

If the USRP cannot be found, make sure that the USRP is plugged-in and restart this program. \
If the problem persists, there may be a problem with you gnuradio installation or USB 2.0.
'''%str(e))		
		
		