"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package Messages
#Handle all of the system messages and error reports.
#@author Josh Blum

from Constants import VERSION,PREFERENCES_FILE_PATH
import traceback

##	A list of functions that can receive a message.	
MESSENGERS_LIST = list()

def register_messenger(messenger):
	"""!
	Append the given messenger to the list of messengers.
	@param messenger a method thats takes a string
	"""
	MESSENGERS_LIST.append(messenger)
	
def send(message):
	"""!
	Give the message to each of the messengers.
	@param message a message string
	"""
	for messenger in MESSENGERS_LIST: messenger(message)
	
###########################################################################
#	Special functions for specific program functionalities
###########################################################################
def send_init():
	send("""<<< Welcome to GRC %s >>>\n"""%VERSION)
					
def send_page_switch(file_path):
	send('\nShowing: "%s"\n'%file_path)

#################	functions for loading flow graphs	########################################					
def send_start_load(file_path):
	send('\nLoading: "%s"'%file_path + '\n')
							
def send_error_load(error):
	send('>>> Error: %s\n'%error)
	traceback.print_exc()

def send_end_load():
	send(">>> Done\n")
	
def send_fail_load(error):
	send('Parser Error: %s\n'%error)
	send(">>> Failue\n")		
	traceback.print_exc()

#################	functions for executing flow graphs	########################################	
def send_start_exec(file_path):
	send('\nExecuting: "%s"'%file_path + '\n')

def send_end_exec(verbose):
	if verbose: 
		send(">>> Verbose:	\n")		
		for verb in verbose: send(verb)
		send("\n")
	send(">>> Done\n")
	
#################	functions for saving flow graphs	########################################		
def send_fail_save(file_path):
	send('>>> Error: Cannot save: %s\n'%file_path)
					
#################	functions for connections	########################################					
def send_fail_connection():
	send('>>> Warning: A connection can only be created between an output socket and an unconnected input socket.\n')					

#################	functions for preferences	########################################
def send_fail_load_preferences():
	send('>>> Error: Cannot load preferences file: "%s"\n'%PREFERENCES_FILE_PATH)
	
def send_fail_save_preferences():
	send('>>> Error: Cannot save preferences file: "%s"\n'%PREFERENCES_FILE_PATH)

