"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package SignalBlockDefs.Audio
#Wrappers and definitions for the audio source and sink.
#@author Josh Blum

from DataTypes import *
from gnuradio import gr

##possible audio rates for audio source/sink
audio_rates = [
	('16KHz', int(16e3)),
	('22.05KHz', int(22.05e3)),
	('24KHz', int(24e3)),
	('32KHz', int(32e3)),
	('44.1KHz', int(44.1e3)),
	('48KHz', int(48e3)),
]

##index of the default audio rate
default_audio_rate_index = 3

#######################################################################################
##	 Defs for Audio Source and Audio Sink
#######################################################################################				
				
def AudioSink(sb):
	from gnuradio import audio #imports
	sb.add_input_socket('lin', Float())
	sb.add_input_socket('rin', Float(), optional=True)
	sb.add_param('Sampling Rate', Enum(audio_rates, default_audio_rate_index), type=True)	
	sb.set_docs('''\
The left input must be connected, The right input is optional. \
If only one audio stream is connected, both channels will receive the stream.''')
	def make(fg, samp_rate):
		return audio.sink(samp_rate.parse())
	return sb, make 
	
def AudioSource(sb):
	from gnuradio import audio	#imports
	sb.add_output_socket('lout', Float())
	sb.add_output_socket('rout', Float(), optional=True)
	sb.add_param('Sampling Rate', Enum(audio_rates, default_audio_rate_index), type=True)
	sb.set_docs('''The left output must be connected, The right output is optional.''')
	def make(fg, samp_rate):
		return audio.source(samp_rate.parse())
	return sb, make 
	
	