"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package SignalBlockDefs.Coders
#Encode and decode.
#@author Josh Blum

from DataTypes import *
from gnuradio import gr

def ConstellationDecoder(sb):
	fcn = gr.constellation_decoder_cb
	sb.add_input_socket('in', Complex())
	sb.add_output_socket('out', Byte())
	sb.add_param('Symbol Position', ComplexVector())
	sb.add_param('Symbol Value Out', ByteVector())
	sb.set_docs('''\
Symbol Position is a vector of complex.
Symbol Value Out is a vector of bytes.''')
	return sb, lambda fg, sym_pos, sym_val: fcn(sym_pos.parse(), sym_val.parse())
		
def DifferentialEncoder(sb):
	fcn = gr.diff_encoder_bb
	sb.add_input_socket('in', Byte())
	sb.add_output_socket('out', Byte())
	sb.add_param('Modulus', Int(10, min=0))
	return sb, lambda fg, mod: fcn(mod.parse())
	
def DifferentialDecoder(sb):
	fcn = gr.diff_decoder_bb
	sb.add_input_socket('in', Byte())
	sb.add_output_socket('out', Byte())
	sb.add_param('Modulus', Int(10, min=0))
	return sb, lambda fg, mod: fcn(mod.parse())
	
def DifferentialPhasor(sb):
	fcn = gr.diff_phasor_cc
	sb.add_input_socket('in', Complex())
	sb.add_output_socket('out', Complex())
	return sb, lambda fg: fcn()
	
def CorrelateAccessCode(sb):
	fcn = gr.correlate_access_code_bb
	sb.add_input_socket('in', Byte())
	sb.add_output_socket('out', Byte())
	return sb, lambda fg: fcn()	
	