"""
Copyright 2007 Free Software Foundation, Inc.
This file is part of GNU Radio

GNU Radio Companion is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GNU Radio Companion is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
"""
##@package SignalBlockDefs.SignalBlockTree
#The data structure to categorize the signal blocks, and to map id tags to block building functions.
#@author Josh Blum

from Elements import SignalBlock
import Sources
import Sinks
import Audio
import GraphicalSinks
import Conversions
import Operators
import Filters
import Modulators
import Misc
import USRP
import Coders
import Trellis
import Packet
import Custom

##A categorized list of all signal blocks
SB_TREE = [
			('Sources', [		
				('Signal Source', Sources.SignalSource),
				('Noise Source', Sources.NoiseSource),								
				('Vector Source', Sources.VectorSource),
				('Random Source', Sources.RandomVector),		
				('Null Source', Sources.NullSource),	
				('File Source', Sources.FileSource),
				('UDP Source', Sources.UDPSource),				
				('Audio Source', Audio.AudioSource),
				('USRP Source', USRP.USRPSource),
				('USRP Dual Source', USRP.USRPDualSource),
				#('RS232 Source', Sources.RS232Source),
			]),
			('Sinks', [		
				('Variable Sink', Sinks.VariableSink),					
				('Null Sink', Sinks.NullSink),
				('File Sink', Sinks.FileSink),	
				('UDP Sink', Sinks.UDPSink),				
				('Audio Sink', Audio.AudioSink),
				('USRP Sink', USRP.USRPSink),
				('USRP Dual Sink', USRP.USRPDualSink),
				#('RS232 Sink', Sinks.RS232Sink),
			]),
			('Graphical Sinks', [			
				('Numerical Sink', GraphicalSinks.NumericalSink),	
				('Scope Sink', GraphicalSinks.ScopeSink),
				('FFT Sink', GraphicalSinks.FFTSink),	
				('Waterfall Sink', GraphicalSinks.WaterfallSink),
				('Constellation Sink', GraphicalSinks.ConstellationSink),
			]),
			('Operators', [							
				('Add', Operators.Add),
				('Subtract', Operators.Subtract),
				('Multiply', Operators.Multiply),				
				('Divide', Operators.Divide),								
				('nLog10', Operators.nLog10),
				('Add Vector', Operators.AddVector),
				('Multiply Vector', Operators.MultiplyVector),
				('Add Constant', Operators.AddConstant),
				('Multiply Constant', Operators.MultiplyConstant),	
				('Add Constant Vector', Operators.AddConstantVector),					
				('Multiply Constant Vector', Operators.MultiplyConstantVector),
			]),
			('Conversions', [			
				('Complex Components', Conversions.ComplexComponents),
				('Complex Conjugate', Conversions.ComplexConjugate),
				('Float to Complex', Conversions.FloatToComplex),
				('Complex to Float', Conversions.ComplexToFloat),
				('Float to Short', Conversions.FloatToShort),
				('Short to Float', Conversions.ShortToFloat),
				('Float to Char', Conversions.FloatToChar),
				('Char to Float', Conversions.CharToFloat),
				('Float to UChar', Conversions.FloatToUChar),
				('UChar to Float', Conversions.UCharToFloat),
				('Complex to IShort', Conversions.ComplexToIShort),
				('IShort to Complex', Conversions.IShortToComplex),	
				('Unpacked to Packed', Conversions.UnpackedToPacked),
				('Packed to Unpacked', Conversions.PackedToUnpacked),
				('Unpack k Bits', Conversions.UnpackKBits),
				('Binary Slicer', Conversions.BinarySlicer),
				('Chunks to Symbols', Conversions.ChunksToSymbols),
				('Map', Conversions.Map),
				('VCO', Conversions.VCO),
				('Interleave', Conversions.Interleave),
				('Deinterleave', Conversions.Deinterleave),
				('Streams to Stream', Conversions.StreamsToStream),
				('Stream to Streams', Conversions.StreamToStreams),
				('Streams to Vector', Conversions.StreamsToVector),
				('Vector to Streams', Conversions.VectorToStreams),
				('Stream to Vector', Conversions.StreamToVector),	
				('Vector to Stream', Conversions.VectorToStream),							
			]),	
			('Generic Filters', [			
				('FIR Filter', Filters.FIRFilter),
				('FFT Filter', Filters.FFTFilter),
				('Freq Xlating FIR Filter', Filters.FreqXlatingFIRFilter),		
				('Rational Resampler', Filters.RationalResampler),	
				('IIR Filer', Filters.IIRFiler),	
				('Filter Delay', Filters.FilterDelay),
				('Channel Model', Filters.ChannelModel),
			]),		
			('Filters', [			
				('Low Pass Filter', Filters.LowPassFilter),
				('High Pass Filter', Filters.HighPassFilter),
				('Band Pass Filter', Filters.BandPassFilter),
				('Band Reject Filter', Filters.BandRejectFilter),
				('Window', Filters.Window),
				('Root Raised Cosine', Filters.RootRaisedCosine),								
				('Single Pole IIR Filter', Filters.SinglePoleIIRFilter),
				('Hilbert', Filters.Hilbert),
				('Goertzel', Filters.Goertzel),	
				('Power Squelch', Filters.PowerSquelch),
				('Downsample', Filters.Downsample),
				('Upsample', Filters.Upsample),				
				('Fractional Resampler', Filters.FractionalResampler),
				('Fractional Interpolator', Filters.FractionalInterpolator),
				('Automatic Gain Control', Filters.AutomaticGainControl),
				('Automatic Gain Control2', Filters.AutomaticGainControl2),
				('Feed Forward AGC', Filters.FeedForwardAGC),
				('CMA Filter', Filters.CMAFilter),
				('Clock Recovery', Filters.ClockRecovery),
				('FFT', Filters.FFT),
				('IFFT', Filters.IFFT),
			]),
			('Modulators', [			
				('Frequency Modulator', Modulators.FrequencyMod),
				('Phase Modulator', Modulators.PhaseMod),
				('Quadrature Demodulator', Modulators.QuadratureDemod),
				('Costas Loop', Modulators.CostasLoop),
				('Phase Locked Loop', Modulators.PLL),
				('WFM Receive', Modulators.WFMReceive),			
				('WFM Transmit', Modulators.WFMTransmit),	
				('NBFM Receive', Modulators.NBFMReceive),			
				('NBFM Transmit', Modulators.NBFMTransmit),	
				('AM Demodulator', Modulators.AMDemod),
				('FM Demodulator', Modulators.FMDemod),
				('PSK Modulator', Modulators.PSKMod),
				('PSK Demodulator', Modulators.PSKDemod),
				('GMSK Modulator', Modulators.GMSKMod),
				('GMSK Demodulator', Modulators.GMSKDemod),
				('QAM Modulator', Modulators.QAMMod),
				('QAM Demodulator', Modulators.QAMDemod),
				('Packet Modulator', Packet.PacketMod),
				('Packet Demodulator', Packet.PacketDemod),
			]),
			('Coders', [
				('Constellation Decoder', Coders.ConstellationDecoder),				
				('Differential Encoder', Coders.DifferentialEncoder),
				('Differential Decoder', Coders.DifferentialDecoder),	
				('Differential Phasor', Coders.DifferentialPhasor),	
				('Correlate Access Code', Coders.CorrelateAccessCode),						
			]),
			('Trellis', [
				('Trellis Encoder', Trellis.Encoder),
				('Metrics', Trellis.Metrics),
				('Viterbi Decoder', Trellis.Viterbi),
				('Viterbi Decoder Combined with Metric', Trellis.Viterbi_Combined),
				('BCJR Algorithm', Trellis.Soft_In_Soft_Out),
				('BCJR Algorithm Combined with Metric', Trellis.Soft_In_Soft_Out_Combined),
				('Interleaver', Trellis.Interleaver),
				('Deinterleaver', Trellis.Deinterleaver),
			]),
			('Misc', [	
				('Throttle', Misc.Throttle),
				('Valve', Misc.Valve),
				('Selector', Misc.Selector),
				('Head', Misc.Head),	
				('Skip Head', Misc.SkipHead),	
				('Input Terminator', Misc.InputTerminator),
				('Copy', Misc.Copy),				
				('Tun Tap', Packet.TunTap),
				('RMS', Misc.RMS),	
				('About', Misc.About),
				('Note', Misc.Note),	
			]),
			('Custom', Custom.CUSTOM_BLOCKS),
		] 
		
def get_signal_block(parent=None, coor=(0,0), rot=0, lookup_tag='', id='', signal_block_constructor=SignalBlock):
	"""!
	Create a new signal block.
	@param parent the flow graph
	@param coor the (x,y) corrdinate
	@param rot the rotation in degrees
	@param lookup_tag the tag of the desired signal block
	@param id the unique id for the new signal block
	@param signal_block_constructor the contructor for a new signal block
	@throw TagNotFoundException tag not found
	@return (the signal block, a gnuradio signal block builder function) tuple
	"""
	sb = signal_block_constructor(parent, coor, rot, lookup_tag, id)
	for category, signal_blocks in SB_TREE:
		for tag, builder in signal_blocks:
			if lookup_tag == tag: return builder(sb)
	raise TagNotFoundException(lookup_tag)	
	
#	remove the bad tags	#
tags_set = set()
for category,tags in SB_TREE:
	tags_to_remove = list()
	for tag in tags:
		try: 
			get_signal_block(lookup_tag=tag[0])
			if tag[0] in tags_set:	#	remove redundant tags	#
				print 'Removing redundant tag "%s" in category "%s"...'%(tag[0], category)
				tags_to_remove.append(tag)	
			else: tags_set.add(tag[0])
		except (ImportError, AttributeError, NameError), e: 
			print e, " in %s! -> continuing..."%tag[0]	
			tags_to_remove.append(tag)		
	for tag in tags_to_remove: tags.remove(tag)
	
#	remove empty categories	#
cats_to_remove = list()
for category,tags in SB_TREE:
	if len(tags) == 0: 
		print 'Removing empty category "%s"...'%category
		cats_to_remove.append((category,tags))
for cat in cats_to_remove: SB_TREE.remove(cat)
	 

class TagNotFoundException(Exception):
	def __init__(self, value):	self.value = value
	def __str__(self): return 'Exception! The tag: %s could not be found'%repr(self.value)
		
def print_sb_tree():
	"""Print the signal block tree."""
	for category,tags in TAGS:
		print category
		for tag in tags: print '\t%s'%tag[0]
					
